@@ -1,5 +1,7 @@
+import ast
+import json
 from datetime import datetime, time
-from typing import Type
+from typing import Type, List
 
 from flask import request, jsonify, abort, current_app, Response
 from permissive_dict import PermissiveDict
@@ -36,11 +38,14 @@ class FlaskSerializeMixin:
     __fs_relationship_fields__ = []
     # add your own converters here
     __fs_column_type_converters__ = {}
-    # add or replace conversion types
-    __fs_convert_types__ = [
-        {"type": bool, "method": lambda v: "y" if v else "n"},
-        {"type": bytes, "method": lambda v: v.encode()},
-    ]
+    # add or replace conversion types to the DB
+    __fs_convert_types__ = {
+        str(bool): lambda v: "y" if v else "n",
+        str(bytes): lambda v: v.encode(),
+        str(dict): lambda v: FlaskSerializeMixin.__fs_json_converter__(v),
+    }
+    __fs_convert_types_original__ = __fs_convert_types__.copy()
+    # types that can be converted to json
     __fs_json_types = [str, dict, list, int, float, bool]
     # default field name when restricting to a particular user
     __fs_user_field__ = "user"
@@ -53,9 +58,36 @@ class FlaskSerializeMixin:
     # previous values of an instance before update attempted
     __fs_previous_field_value__ = {}
     # current version
-    __fs_version__ = "2.0.2"
+    __fs_version__ = "2.1.1"
 
-    def __fs_before_update__(self, data_dict:dict) -> dict:
+    @staticmethod
+    def __fs_json_converter__(value):
+        """
+        convert a json string to a dict using json if required.
+        if encoder error try converting from direct Python
+        If not a string then use the value directly
+
+        :param value:
+        :return:
+        """
+        if value == "":
+            return dict()
+
+        try:
+            j_value = value
+            if type(value) in [str]:
+                try:
+                    j_value = json.loads(value)
+                except json.JSONDecodeError:
+                    j_value = ast.literal_eval(value)
+            elif type(value) not in FlaskSerializeMixin.__fs_json_types:
+                raise Exception(f"unsupported value type: {type(value)}")
+            return j_value
+        except Exception as e:
+            print(f"exception: {e} converting: {value} to JSON")
+            return dict()
+
+    def __fs_before_update__(self, data_dict: dict) -> dict:
         """
         hook to call before any__fs_update_from_dict so that you may alter update values__
         before the item is written in preparation for update to db
@@ -65,7 +97,7 @@ class FlaskSerializeMixin:
         """
         return data_dict
 
-    def __fs_after_commit__(self, create:bool=False):
+    def __fs_after_commit__(self, create: bool = False):
         """
         hook to call after any put/post commit so that you may do something
         self will be the new / updated committed item
@@ -73,7 +105,7 @@ class FlaskSerializeMixin:
         :param create: True when item was just created.
         """
 
-    def __fs_to_date_short__(self, d:datetime)->str:
+    def __fs_to_date_short__(self, d: datetime) -> str:
         """
         convert the given date field to a short date / time without fractional seconds
 
@@ -83,7 +115,7 @@ class FlaskSerializeMixin:
         return str(d).split(".")[0]
 
     @classmethod
-    def fs_query_by_access(cls, user=None, **kwargs)->list:
+    def fs_query_by_access(cls, user=None, **kwargs) -> list:
         """
         filter a query result to that optionally owned by the given user and
         __fs_can_access__()
@@ -143,7 +175,7 @@ class FlaskSerializeMixin:
         if not item or not item.__fs_can_access__():
             return jsonify({})
         return item.fs_as_json
-    
+
     @classmethod
     def fs_json_list(cls, query_result, prop_filters=None):
         """
@@ -175,12 +207,23 @@ class FlaskSerializeMixin:
 
         # ascending
         if cls.__fs_order_by_field__:
-            items = sorted(items, key=lambda i: i[cls.__fs_order_by_field__])
+            if callable(cls.__fs_order_by_field__):
+                items = sorted(
+                    items,
+                    key=cls.__fs_order_by_field__,
+                )
+            else:
+                items = sorted(
+                    items,
+                    key=lambda i: i[cls._fs_get_field_name(cls.__fs_order_by_field__)],
+                )
 
             # descending
         elif cls.__fs_order_by_field_desc__:
             items = sorted(
-                items, key=lambda i: i[cls.__fs_order_by_field_desc__], reverse=True
+                items,
+                key=lambda i: i[cls._fs_get_field_name(cls.__fs_order_by_field_desc__)],
+                reverse=True,
             )
 
         return jsonify(items)
@@ -270,6 +313,8 @@ class FlaskSerializeMixin:
         """
         if isinstance(relationships, FlaskSerializeMixin):
             return relationships.fs_as_dict
+        if isinstance(relationships, str):
+            return relationships
         return [item.fs_as_dict for item in relationships]
 
     @staticmethod
@@ -286,7 +331,43 @@ class FlaskSerializeMixin:
         return value
 
     @staticmethod
-    def __fs_sqlite_date_converter(value):
+    def __fs_sqlite_from_str_json_converter(value):
+        """
+        convert a sqlite json string into a dict
+
+        :param value: string to convert
+        :return: decoded string
+        """
+        if isinstance(value, str):
+            value = json.loads(value)
+
+        if value in ["", None]:
+            return dict()
+
+        return value
+
+    @staticmethod
+    def __fs_sqlite_to_dict_json_converter(value):
+        """
+        convert a sqlite json string from a type that can be json
+        to a dict. if a string do loads
+
+        :param value: string to convert
+        :return: decoded string
+        """
+        if value in ["", None]:
+            return {}
+
+        if type(value) == str:
+            return json.loads(value)
+
+        if type(value) in FlaskSerializeMixin.__fs_json_types:
+            return value
+
+        return {"value": str(value)}
+
+    @staticmethod
+    def __fs_sqlite_to_date_converter(value):
         """
         convert an ISO-9 date or datetime from a form etc into a datetime as sqlite does
         not handle date conversions nicely
@@ -297,12 +378,20 @@ class FlaskSerializeMixin:
         if isinstance(value, datetime) or isinstance(value, time):
             return value
         # assumes ISO 8601 as per javascript standard for dates
-        try:
-            return datetime.strptime(value, "%Y-%m-%dT%H:%M")
-        except:
-            return datetime.strptime(value, "%Y-%m-%d")
+        for date_format in [
+            "%Y-%m-%dT%H:%M:%S",
+            "%Y-%m-%dT%H:%M",
+            "%Y-%m-%d %H:%M:%S",
+            "%Y-%m-%d %H:%M",
+            "%Y-%m-%d",
+        ]:
+            try:
+                return datetime.strptime(value, date_format)
+            except:
+                pass
+        raise Exception(f"could not covert: {value} to datetime")
 
-    def __fs_get_props(self):
+    def _fs_get_props(self):
         """
         get the properties for this table to be used for introspection
 
@@ -325,16 +414,24 @@ class FlaskSerializeMixin:
             }
 
             # SQL columns
-            props.__exclude_fields = [
-                                         "fs_as_dict",
-                                         "fs_as_json",
-                                     ] + self.__fs_exclude_serialize_fields__
+            props.__exclude_fields = ["fs_as_dict", "fs_as_json",] + [
+                self._fs_get_field_name(f) for f in self.__fs_exclude_serialize_fields__
+            ]
             field_list = list(self.__table__.columns)
             if "sqlite" in self.__table__.dialect_options:
                 props.DIALECT = "sqlite"
-                self.__fs_convert_types__.append(
-                    {"type": datetime, "method": self.__fs_sqlite_date_converter}
-                ),
+                self.__fs_convert_types__[
+                    str(datetime)
+                ] = self.__fs_sqlite_to_date_converter
+                self.__fs_convert_types__[
+                    str(dict)
+                ] = self.__fs_sqlite_to_dict_json_converter
+                props.converters["JSON"] = self.__fs_sqlite_from_str_json_converter
+
+            for o, v in self.__fs_convert_types_original__.items():
+                if o not in self.__fs_convert_types__:
+                    self.__fs_convert_types__[o] = v
+
             # detect primary field
             for f in field_list:
                 if f.primary_key:
@@ -350,7 +447,7 @@ class FlaskSerializeMixin:
             ]
             # add relationships
             field_list += [
-                PermissiveDict(name=p, type="RELATIONSHIP")
+                PermissiveDict(name=self._fs_get_field_name(p), type="RELATIONSHIP")
                 for p in self.__fs_relationship_fields__
             ]
             # add custom converters
@@ -365,8 +462,8 @@ class FlaskSerializeMixin:
                     if not f.converter:
                         # any non json supported types gets a str
                         if (
-                                getattr(f.type, "python_type", None)
-                                not in self.__fs_json_types
+                            getattr(f.type, "python_type", None)
+                            not in self.__fs_json_types
                         ):
                             f.converter = str
                     props.field_list.append(f)
@@ -374,22 +471,22 @@ class FlaskSerializeMixin:
             self.__fs_model_props[self.__table__] = props
         return props
 
-    def __fs_get_fields(self):
+    def _fs_get_fields(self) -> List[str]:
         """
         return a list of field objects that are valid
         using __fs_private_field__
         [{name,c__type,converter},...]
 
-        :return:
+        :return: list of strings
         """
         fields = []
-        for c in self.__fs_get_props().field_list:
+        for c in self._fs_get_props().field_list:
             if not self.__fs_private_field__(c.name):
                 fields.append(c)
         return fields
 
     @property
-    def fs_as_dict(self):
+    def fs_as_dict(self) -> dict:
         """
         convert a sql alchemy query result item to dict
         override these properties to control the result:
@@ -397,14 +494,14 @@ class FlaskSerializeMixin:
         * __fs_relationship_fields__ - follow relationships listed
         * __fs_exclude_serialize_fields__ - exclude listed fields from serialization
         * __fs_column_type_converters__ - add additional sql column type converters to DATETIME, PROPERTY and RELATIONSHIP
-        :return {dictionary} the item as a dict
+        :return {dict} the item as a dict
         """
 
         # built in converters
         # can be replaced using __fs_column_type_converters__
         d = {}
 
-        for c in self.__fs_get_fields():
+        for c in self._fs_get_fields():
             try:
                 d[c.name] = v = getattr(self, c.name, "")
             except Exception as e:
@@ -426,61 +523,79 @@ class FlaskSerializeMixin:
         """
         get the type of the update to db field from cached table properties
 
-        :param field:
+        :param field: the field to lookup
+        :param value: unused
         :return: class of the type
         """
-        props = self.__fs_get_props()
+        props = self._fs_get_props()
         if props:
             for f in props.field_list:
                 if f.name == field:
                     if (
-                            f.c_type.startswith("VARCHAR")
-                            or f.c_type.startswith("CHAR")
-                            or f.c_type.startswith("TEXT")
+                        f.c_type.startswith("VARCHAR")
+                        or f.c_type.startswith("CHAR")
+                        or f.c_type.startswith("TEXT")
                     ):
                         return str
                     if f.c_type.startswith("INTEGER"):
                         return int
                     if (
-                            f.c_type.startswith("FLOAT")
-                            or f.c_type.startswith("REAL")
-                            or f.c_type.startswith("NUMERIC")
+                        f.c_type.startswith("FLOAT")
+                        or f.c_type.startswith("REAL")
+                        or f.c_type.startswith("NUMERIC")
                     ):
                         return float
                     if f.c_type.startswith("DATE") or f.c_type.startswith("TIME"):
                         return datetime
                     if f.c_type.startswith("BOOLEAN"):
                         return bool
+                    if "JSON" in f.c_type:
+                        return dict
                     if "LOB" in f.c_type:
                         return bytes
 
         return None
 
-    def __fs_convert_value(self, name, value):
+    def __fs_convert_value_to_db_suitable_value(self, name, value):
         """
         convert the value based upon type to a representation suitable for saving to the db
         override built in conversions by setting the value of __fs_convert_types__.
         First uses bare value to determine type and then uses db derived values
         ie:
-        __fs_convert_types__ = [{'type':bool, 'method': lambda x: not x}]
+        __fs_convert_types__ = {str(bool): lambda x: not x}
 
         :param name: name of the field to update
         :param value: value to update with
         :return: the converted value
         """
-        for t in self.__fs_convert_types__:
-            if isinstance(value, t["type"]):
-                value = t["method"](value)
-                return value
+        lookup_key = str(type(value))
+        if lookup_key in self.__fs_convert_types__:
+            value = self.__fs_convert_types__[lookup_key](value)
+            return value
 
         instance_type = self.__fs_get_update_field_type(name, value)
         if instance_type:
-            for t in self.__fs_convert_types__:
-                if instance_type == t["type"]:
-                    value = t["method"](value)
-                    return value
+            lookup_key = str(instance_type)
+            if lookup_key in self.__fs_convert_types__:
+                value = self.__fs_convert_types__[lookup_key](value)
+                return value
+
         return value
 
+    @classmethod
+    def _fs_get_field_name(cls, field) -> str:
+        """
+        return the column name of the field type or string
+
+        :param field: field to be looked up
+        :return: str
+        """
+        if isinstance(field, str):
+            return field
+        if cls.db and isinstance(field, cls.db.Column):
+            return field.name
+        return str(field).split(".")[-1]
+
     @classmethod
     def fs_request_create_form(cls, **kwargs):
         """
@@ -494,12 +609,13 @@ class FlaskSerializeMixin:
 
         new_item = cls(**kwargs)
 
-        __fs_create_fields__ = list(new_item.__fs_create_fields__)
-        if len(__fs_create_fields__ or "") == 0:
-            __fs_create_fields__ = [
+        fs_create_fields = list(new_item.__fs_create_fields__)
+        if len(fs_create_fields or "") == 0:
+            fs_create_fields = [
                 c.name
-                for c in new_item.__fs_get_fields()
-                if isinstance(c, cls.db.Column) and c.name != new_item.__fs_get_props().primary_key_field
+                for c in new_item._fs_get_fields()
+                if isinstance(c, cls.db.Column)
+                and c.name != new_item._fs_get_props().primary_key_field
             ]
 
         try:
@@ -508,12 +624,15 @@ class FlaskSerializeMixin:
             json_data = request.form
 
         if len(json_data) > 0:
-            for field in __fs_create_fields__:
-                if cls.db and isinstance(field, cls.db.Column):
-                    field = field.name
+            for field in fs_create_fields:
+                field = cls._fs_get_field_name(field)
                 if field in json_data:
                     value = json_data.get(field)
-                    setattr(new_item, field, new_item.__fs_convert_value(field, value))
+                    setattr(
+                        new_item,
+                        field,
+                        new_item.__fs_convert_value_to_db_suitable_value(field, value),
+                    )
 
         new_item.__fs_verify__(create=True)
         new_item.__fs_update_timestamp__()
@@ -522,11 +641,11 @@ class FlaskSerializeMixin:
         new_item.__fs_after_commit__(create=True)
         return new_item
 
-    def __fs_request_update(self, json_data):
+    def __fs_request_update(self, json_data: dict) -> bool:
         """
         update the current db object
-
-        :return:
+        :param json_data:
+        :return: boolean
         """
         if not self.__fs_can_update__():
             return False
@@ -579,7 +698,7 @@ class FlaskSerializeMixin:
             if hasattr(self, field):
                 setattr(self, field, self.__fs_timestamp_stamper__())
 
-    def fs_update_from_dict(self, data_dict):
+    def fs_update_from_dict(self, data_dict: dict) -> None:
         """
         uses a dict to update fields of the model instance.  sets previous values to
         self.previous_values[field_name] before the update
@@ -592,16 +711,21 @@ class FlaskSerializeMixin:
         if len(self.__fs_update_fields__ or "") == 0:
             __fs_update_fields__ = [
                 c.name
-                for c in self.__fs_get_fields()
+                for c in self._fs_get_fields()
                 if isinstance(c, self.db.Column)
-                   and c.name != self.__fs_get_props().primary_key_field
+                and c.name != self._fs_get_props().primary_key_field
             ]
         for field in __fs_update_fields__:
-            if self.db and isinstance(field, self.db.Column):
-                field = field.name
+            field = self._fs_get_field_name(field)
             self.__fs_previous_field_value__[field] = getattr(self, field)
             if field in data_dict:
-                setattr(self, field, self.__fs_convert_value(field, data_dict[field]))
+                setattr(
+                    self,
+                    field,
+                    self.__fs_convert_value_to_db_suitable_value(
+                        field, data_dict[field]
+                    ),
+                )
 
     def __fs_can_access__(self):
         """
@@ -699,7 +823,8 @@ class FlaskSerializeMixin:
                 return item.fs_as_json
 
             elif request.method == "POST" or request.method == "PUT":
-                # update single item
+                # update single item with locked row
+                item = cls.query.with_for_update(of=cls).get_or_404(item_id)
                 if item.fs_request_update_form():
                     return jsonify(
                         dict(
@@ -708,6 +833,7 @@ class FlaskSerializeMixin:
                             properties=item.__fs_return_properties(),
                         )
                     )
+                cls.db.session.rollback()
                 return Response("UPDATE forbidden", 403)
 
             elif request.method == "DELETE":
