@@ -17,7 +17,10 @@ from wtforms import (
     SubmitField,
 )
 
-from flask_serialize.flask_serialize import FlaskSerialize
+from flask_serialize.flask_serialize import (
+    FlaskSerialize,
+    FlaskSerializeMixin,
+)
 from flask_serialize.form_page import FormPageMixin
 
 app = Flask("test_app")
@@ -28,9 +31,9 @@ app.config["SQLALCHEMY_DATABASE_URI"] = os.environ.get(
 )
 db = SQLAlchemy(app)
 migrate = Migrate(app, db)
+FlaskSerializeMixin.db = db
 fs_mixin = FlaskSerialize(db)
 
-
 # =========================
 # TINY TEST FLASK APP
 # =========================
@@ -51,6 +54,7 @@ class EditForm(FlaskForm):
     value = StringField("value")
     number = IntegerField("number")
     deci = DecimalField("deci")
+    j = StringField("j")
     submit = SubmitField()
 
 
@@ -168,7 +172,7 @@ def route_setting_update_post(item_id):
     item = Setting.query.get_or_404(item_id)
     try:
         if not item.fs_request_update_form():
-            return Response('update not allowed', 403)
+            return Response("update not allowed", 403)
     except Exception as e:
         print(e)
         return Response("POST Error updating item: " + str(e), 500)
@@ -250,7 +254,7 @@ def route_user(item_id=None):
 
 @app.route("/user_add_data/<int:item_id>", methods=["POST"])
 def route_user_add_data(item_id):
-    value = request.form.get('data', '')
+    value = request.form.get("data", "")
     user = User.query.get_or_404(item_id)
     user_data = UserData(user=user, value=value)
     db.session.add(user_data)
@@ -279,12 +283,9 @@ class SubSetting(fs_mixin, db.Model):
         "flong",
         "boolean",
     ]
-    __fs_convert_types__ = [
-        {
-            "type": bool,
-            "method": lambda v: (type(v) == bool and v) or str(v).lower() == "true",
-        },
-    ]
+    __fs_convert_types__ = {
+        str(bool): lambda v: (type(v) == bool and v) or str(v).lower() == "true"
+    }
 
     @staticmethod
     def one_day_ago():
@@ -313,7 +314,7 @@ class User(fs_mixin, db.Model):
         "UserData", backref="user", cascade="all, delete-orphan"
     )
 
-    __fs_relationship_fields__ = ['data_items']
+    __fs_relationship_fields__ = ["data_items"]
 
 
 class UserData(fs_mixin, db.Model):
@@ -335,7 +336,7 @@ class DateTest(fs_mixin, db.Model):
     __fs_update_fields__ = __fs_create_fields__ = ["a_date"]
 
 
-class Setting(fs_mixin, FormPageMixin, db.Model):
+class Setting(FlaskSerializeMixin, FormPageMixin, db.Model):
     id = db.Column(db.Integer, primary_key=True)
 
     setting_type = db.Column(db.String(120), index=True, default="misc")
@@ -350,6 +351,7 @@ class Setting(fs_mixin, FormPageMixin, db.Model):
     floaty = db.Column(db.Float(), default=1.1)
     deci = db.Column(db.Numeric(2), default=0.0)
     # converter fields
+    j = db.Column(db.JSON, default={"number_1": 1, "bool_true": True})
     lob = db.Column(db.LargeBinary())
     # relationships
     sub_settings = db.relationship(
@@ -366,10 +368,11 @@ class Setting(fs_mixin, FormPageMixin, db.Model):
         "key",
         "active",
         "number",
-        "floaty",
+        floaty,
         "scheduled",
         "deci",
         "lob",
+        j,
     ]
     __fs_create_fields__ = [
         setting_type,
@@ -381,32 +384,27 @@ class Setting(fs_mixin, FormPageMixin, db.Model):
         "number",
         "deci",
         "lob",
+        j,
     ]
-    __fs_exclude_serialize_fields__ = ["created"]
+
+    __fs_exclude_serialize_fields__ = [created]
     __fs_exclude_json_serialize_fields__ = ["updated"]
-    __fs_relationship_fields__ = ["sub_settings", "single"]
+    __fs_relationship_fields__ = [sub_settings, "single"]
     __fs_update_properties__ = ["prop_test"]
     __fs_order_by_field__ = "value"
     # lob
     __fs_column_type_converters__ = {"LOB": lambda v: str(v)}
     # convert types
     __fs_scheduled_date_format__ = "%Y-%m-%d %H:%M:%S"
-    __fs_convert_types__ = [
-        {
-            "type": bool,
-            "method": lambda v: "y"
-            if (type(v) == bool and v) or str(v).lower() == "true"
-            else "n",
-        },
-        {"type": int, "method": lambda n: int(n) * 2},
-        {
-            "type": datetime,
-            "method": lambda n: datetime.strptime(
-                n, Setting.__fs_scheduled_date_format__
-            ),
-        },
-        {"type": bytes, "method": lambda v: v.encode()},
-    ]
+    __fs_convert_types__ = {
+        str(bool): lambda v: "y"
+        if (type(v) == bool and v) or str(v).lower() == "true"
+        else "n",
+        str(int): lambda n: int(n) * 2,
+        str(datetime): lambda n: datetime.strptime(
+            n, Setting.__fs_scheduled_date_format__
+        ),
+    }
     # form_page
     form_page_form = EditForm
     form_page_route_update = "route_setting_form"
@@ -438,7 +436,7 @@ class Setting(fs_mixin, FormPageMixin, db.Model):
 
     # checks if Flask-Serialize can access
     def __fs_can_update__(self):
-        if self.value == '9999':
+        if self.value == "9999":
             return False
         if not self.__fs_can_access__():
             raise Exception("Update not allowed.  Magic value!")
@@ -523,7 +521,7 @@ class SimpleModel(fs_mixin, db.Model):
 class BadModel(fs_mixin, db.Model):
     id = db.Column(db.Integer, primary_key=True)
     value = db.Column(db.String(30), default="")
-    __fs_column_type_converters__ = {'VARCHAR': lambda x: x / 0}
+    __fs_column_type_converters__ = {"VARCHAR": lambda x: x / 0}
 
     def __repr__(self):
         return "<BadModel %r>" % (self.value)
